@@ -113,6 +113,13 @@ struct fileinlist {
   struct filenamenode *namenode;
 };
 
+/**
+ * Queue of filenamenode entries.
+ */
+struct filenamenode_queue {
+  struct fileinlist *head, **tail;
+};
+
 /**
  * When we deal with an ‘overridden’ file, every package except the
  * overriding one is considered to contain the other file instead. Both
